@@ -1,25 +1,37 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (c) 2003 Cluster File Systems, Inc.
+ * GPL HEADER START
  *
- *   This file is part of the Lustre file system, http://www.lustre.org
- *   Lustre is a trademark of Cluster File Systems, Inc.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   You may have signed or agreed to another license before downloading
- *   this software.  If so, you are bound by the terms and conditions
- *   of that agreement, and the following does not apply to you.  See the
- *   LICENSE file included with this distribution for more information.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   If you did not agree to a different license, then this copy of Lustre
- *   is open source software; you can redistribute it and/or modify it
- *   under the terms of version 2 of the GNU General Public License as
- *   published by the Free Software Foundation.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   In either case, Lustre is distributed in the hope that it will be
- *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
- *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   license text for more details.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
  */
 
 #define DEBUG_SUBSYSTEM S_MDS
@@ -34,11 +46,7 @@
 #include <linux/stat.h>
 #include <linux/errno.h>
 #include <linux/version.h>
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
-# include <linux/locks.h>   // for wait_on_buffer
-#else
-# include <linux/buffer_head.h>   // for wait_on_buffer
-#endif
+#include <linux/buffer_head.h>   // for wait_on_buffer
 #include <linux/unistd.h>
 
 #include <asm/system.h>
@@ -82,8 +90,8 @@ void mds_pack_inode2body(struct mds_body *b, struct inode *inode)
         b->uid = inode->i_uid;
         b->gid = inode->i_gid;
         b->flags = (b->flags & MDS_BFLAG_EXT_FLAGS) |
-                   ll_inode_to_ext_flags(inode->i_flags,
-                                         !(b->flags & MDS_BFLAG_EXT_FLAGS));
+                    ll_inode_to_ext_flags(inode->i_flags,
+                                          !(b->flags & MDS_BFLAG_EXT_FLAGS));
         b->rdev = inode->i_rdev;
         /* Return the correct link count for orphan inodes */
         b->nlink = mds_inode_is_orphan(inode) ? 0 : inode->i_nlink;
@@ -144,7 +152,7 @@ static int mds_setattr_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->sa_fsuid;
         r->ur_uc.luc_fsgid = rec->sa_fsgid;
-        r->ur_uc.luc_cap = rec->sa_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->sa_cap);
         r->ur_uc.luc_suppgid1 = rec->sa_suppgid;
         r->ur_uc.luc_suppgid2 = -1;
         r->ur_fid1 = &rec->sa_fid;
@@ -194,7 +202,7 @@ static int mds_create_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->cr_fsuid;
         r->ur_uc.luc_fsgid = rec->cr_fsgid;
-        r->ur_uc.luc_cap = rec->cr_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->cr_cap);
         r->ur_uc.luc_suppgid1 = rec->cr_suppgid;
         r->ur_uc.luc_suppgid2 = -1;
         r->ur_fid1 = &rec->cr_fid;
@@ -247,7 +255,7 @@ static int mds_link_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->lk_fsuid;
         r->ur_uc.luc_fsgid = rec->lk_fsgid;
-        r->ur_uc.luc_cap = rec->lk_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->lk_cap);
         r->ur_uc.luc_suppgid1 = rec->lk_suppgid1;
         r->ur_uc.luc_suppgid2 = rec->lk_suppgid2;
         r->ur_fid1 = &rec->lk_fid1;
@@ -282,7 +290,7 @@ static int mds_unlink_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->ul_fsuid;
         r->ur_uc.luc_fsgid = rec->ul_fsgid;
-        r->ur_uc.luc_cap = rec->ul_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->ul_cap);
         r->ur_uc.luc_suppgid1 = rec->ul_suppgid;
         r->ur_uc.luc_suppgid2 = -1;
         r->ur_mode = rec->ul_mode;
@@ -319,7 +327,7 @@ static int mds_rename_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->rn_fsuid;
         r->ur_uc.luc_fsgid = rec->rn_fsgid;
-        r->ur_uc.luc_cap = rec->rn_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->rn_cap);
         r->ur_uc.luc_suppgid1 = rec->rn_suppgid1;
         r->ur_uc.luc_suppgid2 = rec->rn_suppgid2;
         r->ur_fid1 = &rec->rn_fid1;
@@ -360,7 +368,7 @@ static int mds_open_unpack(struct ptlrpc_request *req, int offset,
 
         r->ur_uc.luc_fsuid = rec->cr_fsuid;
         r->ur_uc.luc_fsgid = rec->cr_fsgid;
-        r->ur_uc.luc_cap = rec->cr_cap;
+        cfs_kernel_cap_unpack(&r->ur_uc.luc_cap, rec->cr_cap);
         r->ur_uc.luc_suppgid1 = rec->cr_suppgid;
         r->ur_uc.luc_suppgid2 = -1;
         r->ur_fid1 = &rec->cr_fid;
@@ -413,7 +421,7 @@ int mds_update_unpack(struct ptlrpc_request *req, int offset,
                 RETURN(-EFAULT);
 
         opcode = *opcodep;
-        if (lustre_msg_swabbed(req->rq_reqmsg))
+        if (lustre_req_need_swab(req))
                 __swab32s(&opcode);
 
         if (opcode >= REINT_MAX || mds_unpackers[opcode] == NULL) {
@@ -428,7 +436,7 @@ int mds_update_unpack(struct ptlrpc_request *req, int offset,
 }
 
 void mds_root_squash(struct mds_obd *mds, lnet_nid_t *peernid,
-                     __u32 *fsuid, __u32 *fsgid, __u32 *cap,
+                     __u32 *fsuid, __u32 *fsgid, cfs_kernel_cap_t *kcap,
                      __u32 *suppgid, __u32 *suppgid2)
 {
         if (!mds->mds_squash_uid || *fsuid)
@@ -437,13 +445,13 @@ void mds_root_squash(struct mds_obd *mds, lnet_nid_t *peernid,
         if (*peernid == mds->mds_nosquash_nid)
                 return;
 
-        CDEBUG(D_OTHER, "squash req from %s, (%d:%d/%x)=>(%d:%d/%x)\n",
-               libcfs_nid2str(*peernid), *fsuid, *fsgid, *cap,
-               mds->mds_squash_uid, mds->mds_squash_gid, 0);
+        CDEBUG(D_OTHER, "squash req from %s, (%d:%d)=>(%d:%d)\n",
+               libcfs_nid2str(*peernid), *fsuid, *fsgid,
+               mds->mds_squash_uid, mds->mds_squash_gid);
 
         *fsuid = mds->mds_squash_uid;
         *fsgid = mds->mds_squash_gid;
-        *cap = 0;
+        cfs_kernel_cap_unpack(kcap, 0);
         *suppgid = -1;
         if (suppgid2)
                 *suppgid2 = -1;
@@ -467,13 +475,13 @@ int mds_init_ucred(struct lvfs_ucred *ucred, struct ptlrpc_request *req,
         } else
 #endif
         {
+                cfs_kernel_cap_unpack(&ucred->luc_cap, body->capability);
                 mds_root_squash(mds, &req->rq_peer.nid, &body->fsuid,
-                                &body->fsgid, &body->capability,
+                                &body->fsgid, &ucred->luc_cap,
                                 &body->suppgid, NULL);
 
                 ucred->luc_fsuid = body->fsuid;
                 ucred->luc_fsgid = body->fsgid;
-                ucred->luc_cap = body->capability;
         }
 
         ucred->luc_uce = upcall_cache_get_entry(mds->mds_group_hash,
