@@ -1,29 +1,44 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (c) 2003 Cluster File Systems, Inc.
- *   Author: Peter Braam <braam@clusterfs.com>
- *   Author: Andreas Dilger <adilger@clusterfs.com>
- *   Author: Phil Schwan <phil@clusterfs.com>
- *   Author: Mike Shaver <shaver@clusterfs.com>
+ * GPL HEADER START
  *
- *   This file is part of the Lustre file system, http://www.lustre.org
- *   Lustre is a trademark of Cluster File Systems, Inc.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   You may have signed or agreed to another license before downloading
- *   this software.  If so, you are bound by the terms and conditions
- *   of that agreement, and the following does not apply to you.  See the
- *   LICENSE file included with this distribution for more information.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   If you did not agree to a different license, then this copy of Lustre
- *   is open source software; you can redistribute it and/or modify it
- *   under the terms of version 2 of the GNU General Public License as
- *   published by the Free Software Foundation.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   In either case, Lustre is distributed in the hope that it will be
- *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
- *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   license text for more details.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/mds/mds_open.c
+ *
+ * Author: Peter Braam <braam@clusterfs.com>
+ * Author: Andreas Dilger <adilger@clusterfs.com>
+ * Author: Phil Schwan <phil@clusterfs.com>
+ * Author: Mike Shaver <shaver@clusterfs.com>
  */
 
 #ifndef EXPORT_SYMTAB
@@ -35,12 +50,8 @@
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/version.h>
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
-# include <linux/buffer_head.h>
-# include <linux/workqueue.h>
-#else
-# include <linux/locks.h>
-#endif
+#include <linux/buffer_head.h>
+#include <linux/workqueue.h>
 
 #include <obd_class.h>
 #include <obd_lov.h>
@@ -356,11 +367,11 @@ static int mds_create_objects(struct ptlrpc_request *req, int offset,
                 rc = fsfilt_set_md(obd, inode, *handle, lmm, lmm_size, "lov");
                 if (rc)
                         CERROR("open replay failed to set md:%d\n", rc);
-                lmm_buf = lustre_msg_buf(req->rq_repmsg, offset, lmm_size);
-                LASSERT(lmm_buf);
-                memcpy(lmm_buf, lmm, lmm_size);
 
-                *objid = lmm_buf;
+                /* for replay we not need send lmm to client, this not used now */
+                lustre_shrink_reply(req, offset, 0, 1);
+                *objid = lmm;
+
                 RETURN(rc);
         }
 
@@ -380,7 +391,6 @@ static int mds_create_objects(struct ptlrpc_request *req, int offset,
 
         obdo_from_inode(oinfo.oi_oa, inode, OBD_MD_FLTYPE | OBD_MD_FLATIME |
                         OBD_MD_FLMTIME | OBD_MD_FLCTIME);
-
         if (!(rec->ur_flags & MDS_OPEN_HAS_OBJS)) {
                 /* check if things like lfs setstripe are sending us the ea */
                 if (rec->ur_flags & MDS_OPEN_HAS_EA) {
@@ -390,22 +400,23 @@ static int mds_create_objects(struct ptlrpc_request *req, int offset,
                         if (rc)
                                 GOTO(out_oa, rc);
                 } else {
-                        OBD_ALLOC(lmm, mds->mds_max_mdsize);
+                        __u32 lmm_sz = mds->mds_max_mdsize;
+                        OBD_ALLOC(lmm, lmm_sz);
                         if (lmm == NULL)
                                 GOTO(out_oa, rc = -ENOMEM);
 
-                        lmm_size = mds->mds_max_mdsize;
+                        lmm_size = lmm_sz;
                         rc = mds_get_md(obd, dchild->d_parent->d_inode,
                                         lmm, &lmm_size, 1, 0);
                         if (rc > 0)
                                 rc = obd_iocontrol(OBD_IOC_LOV_SETSTRIPE,
                                                    mds->mds_osc_exp,
                                                    0, &oinfo.oi_md, lmm);
-                        OBD_FREE(lmm, mds->mds_max_mdsize);
+                        OBD_FREE(lmm, lmm_sz);
                         if (rc)
                                 GOTO(out_oa, rc);
                 }
-                rc = obd_create(mds->mds_osc_exp, oinfo.oi_oa, 
+                rc = obd_create(mds->mds_osc_exp, oinfo.oi_oa,
                                 &oinfo.oi_md, &oti);
                 if (rc) {
                         int level = D_ERROR;
@@ -439,8 +450,10 @@ static int mds_create_objects(struct ptlrpc_request *req, int offset,
                 oinfo.oi_oa->o_fid = body->fid1.id;
                 oinfo.oi_oa->o_generation = body->fid1.generation;
                 oinfo.oi_oa->o_valid |= OBD_MD_FLFID | OBD_MD_FLGENER;
+                oinfo.oi_policy.l_extent.start = i_size_read(inode);
+                oinfo.oi_policy.l_extent.end = OBD_OBJECT_EOF;
 
-                rc = obd_setattr_rqset(mds->mds_osc_exp, &oinfo, &oti);
+                rc = obd_punch_rqset(mds->mds_osc_exp, &oinfo, &oti);
                 if (rc) {
                         CERROR("error setting attrs for inode %lu: rc %d\n",
                                inode->i_ino, rc);
@@ -496,10 +509,11 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
                              struct lustre_handle *child_lockh)
 {
         struct mds_export_data *med = &req->rq_export->exp_mds_data;
-        struct mds_client_data *mcd = med->med_mcd;
+        struct lsd_client_data *lcd = med->med_lcd;
         struct mds_obd *mds = mds_req2mds(req);
         struct mds_file_data *mfd;
-        struct obd_device *obd = req->rq_export->exp_obd;
+        struct obd_export *exp = req->rq_export;
+        struct obd_device *obd = exp->exp_obd;
         struct dentry *parent, *dchild;
         struct ldlm_reply *rep;
         struct mds_body *body;
@@ -513,8 +527,8 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
         body = lustre_msg_buf(req->rq_repmsg, DLM_REPLY_REC_OFF, sizeof(*body));
 
         /* copy rc, transno and disp; steal locks */
-        mds_req_from_mcd(req, mcd);
-        intent_set_disposition(rep, le32_to_cpu(mcd->mcd_last_data));
+        mds_req_from_lcd(req, lcd);
+        intent_set_disposition(rep, le32_to_cpu(lcd->lcd_last_data));
 
         /* Only replay if create or open actually happened. */
         if (!intent_disposition(rep, DISP_OPEN_CREATE | DISP_OPEN_OPEN) ) {
@@ -523,10 +537,31 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
         }
 
         parent = mds_fid2dentry(mds, rec->ur_fid1, NULL);
-        LASSERT(!IS_ERR(parent));
+        if (IS_ERR(parent)) {
+                rc = PTR_ERR(parent);
+                LCONSOLE_WARN("Parent "LPU64"/%u lookup error %d."
+                              " Evicting client %s with export %s.\n",
+                              rec->ur_fid1->id, rec->ur_fid1->generation, rc,
+                              obd_uuid2str(&exp->exp_client_uuid),
+                              obd_export_nid2str(exp));
+                mds_export_evict(exp);
+                EXIT;
+                return;
+        }
 
         dchild = ll_lookup_one_len(rec->ur_name, parent, rec->ur_namelen - 1);
-        LASSERT(!IS_ERR(dchild));
+        if (IS_ERR(dchild)) {
+                rc = PTR_ERR(dchild);
+                LCONSOLE_WARN("Child "LPU64"/%u lookup error %d."
+                              " Evicting client %s with export %s.\n",
+                              rec->ur_fid1->id, rec->ur_fid1->generation, rc,
+                              obd_uuid2str(&exp->exp_client_uuid),
+                              obd_export_nid2str(exp));
+                mds_export_evict(exp);
+                l_dput(parent);
+                EXIT;
+                return;
+        }
 
         if (!dchild->d_inode)
                 GOTO(out_dput, 0); /* child not present to open */
@@ -609,9 +644,16 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
          * to detect a re-open */
         if (mfd == NULL) {
                 if (rec->ur_flags & MDS_OPEN_JOIN_FILE) {
+#if LUSTRE_FIX >= 50
+                        /* Allow file join in beta builds to allow debugging */
                         rc = mds_join_file(rec, req, dchild, NULL);
                         if (rc)
                                 GOTO(out_dput, rc);
+#else
+                        CWARN("file join is not supported in this version of "
+                              "Lustre\n");
+                        GOTO(out_dput, req->rq_status = rc = -EOPNOTSUPP);
+#endif
                 }
                 mntget(mds->mds_vfsmnt);
                 CERROR("Re-opened file \n");
@@ -634,6 +676,16 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
 
         mds_mfd_put(mfd);
 
+        if (!intent_disposition(rep, DISP_OPEN_LOCK))
+                GOTO(out_dput, 0);
+
+        /* child_lockh has been set in fixup_handle_for_resent_req called
+         * in mds_intent_policy for resent request */
+        if (child_lockh == NULL || !lustre_handle_is_used(child_lockh)) {
+                /* the lock is already canceled! clear DISP_OPEN_LOCK */
+                intent_disposition(rep, ~DISP_OPEN_LOCK);
+        }
+
  out_dput:
         if (put_child)
                 l_dput(dchild);
@@ -695,11 +747,18 @@ static int mds_finish_open(struct ptlrpc_request *req, struct dentry *dchild,
                         RETURN(-EEXIST);
                 }
                 if (rec->ur_flags & MDS_OPEN_JOIN_FILE) {
+#if LUSTRE_FIX >= 50
+                        /* Allow file join in beta builds to allow debugging */
                         UNLOCK_INODE_MUTEX(dchild->d_inode);
                         rc = mds_join_file(rec, req, dchild, lockh);
                         if (rc)
                                 RETURN(rc);
                         LOCK_INODE_MUTEX(dchild->d_inode);
+#else
+                        CWARN("file join is not supported in this version of "
+                              "Lustre\n");
+                        RETURN(-EOPNOTSUPP);
+#endif
                 }
                 if (!(body->valid & OBD_MD_FLEASIZE) &&
                     !(body->valid & OBD_MD_FLMODEASIZE)) {
@@ -737,6 +796,9 @@ static int mds_finish_open(struct ptlrpc_request *req, struct dentry *dchild,
                         RETURN(rc);
         }
 
+        if ((rc = mds_lov_prepare_objids(obd,lmm)) != 0)
+                RETURN(rc);
+
         intent_set_disposition(rep, DISP_OPEN_OPEN);
         mfd = mds_dentry_open(dchild, mds->mds_vfsmnt, flags, req);
         if (IS_ERR(mfd))
@@ -747,7 +809,7 @@ static int mds_finish_open(struct ptlrpc_request *req, struct dentry *dchild,
 
         mds_lov_update_objids(obd, lmm);
 
-        if (rc) /* coverity[deadcode] */
+        if (rc)
                 mds_mfd_unlink(mfd, 1);
 
         mds_mfd_put(mfd);
@@ -873,10 +935,9 @@ int mds_open(struct mds_update_record *rec, int offset,
         int child_mode = LCK_CR;
         /* Always returning LOOKUP lock if open succesful to guard
            dentry on client. */
-        ldlm_policy_data_t policy = {.l_inodebits={MDS_INODELOCK_LOOKUP}};
-        struct ldlm_res_id child_res_id = { .name = {0}};
         int lock_flags = 0;
         int rec_pending = 0;
+        int use_parent, need_open_lock;
         unsigned int gid = current->fsgid;
         ENTRY;
 
@@ -950,17 +1011,45 @@ int mds_open(struct mds_update_record *rec, int offset,
                 RETURN(-ENOMEM);
         }
 
-        /* Step 1: Find and lock the parent */
         if (rec->ur_flags & (MDS_OPEN_CREAT | MDS_OPEN_JOIN_FILE))
                 parent_mode = LCK_EX;
-        dparent = mds_fid2locked_dentry(obd, rec->ur_fid1, NULL, parent_mode,
-                                        &parent_lockh, rec->ur_name,
-                                        rec->ur_namelen - 1,
-                                        MDS_INODELOCK_UPDATE);
-        if (IS_ERR(dparent)) {
-                rc = PTR_ERR(dparent);
+
+        /* We cannot use acc_mode here, because it is zeroed in case of
+           creating a file, so we get wrong lockmode */
+        if (rec->ur_flags & FMODE_WRITE)
+                child_mode = LCK_CW;
+        else if (rec->ur_flags & MDS_FMODE_EXEC)
+                child_mode = LCK_PR;
+        else
+                child_mode = LCK_CR;
+
+        /* join file and nfsd can't need lookup dchild as use parent for it */
+        use_parent = (!(lustre_msg_get_flags(req->rq_reqmsg) & MSG_REPLAY) &&
+                     (rec->ur_flags & MDS_OPEN_LOCK) && (rec->ur_namelen == 1)) ||
+                     (rec->ur_flags & MDS_OPEN_JOIN_FILE);
+
+        need_open_lock = !(lustre_msg_get_flags(req->rq_reqmsg) & MSG_REPLAY) &&
+                           (rec->ur_flags & MDS_OPEN_LOCK);
+
+        /* Try to lock both parent and child first. If child is not found,
+         * return only locked parent.  This is enough to prevent other
+         * threads from changing this directory until creation is finished. */
+        rc = mds_get_parent_child_locked(obd, &obd->u.mds,
+                                         rec->ur_fid1,
+                                         &parent_lockh,
+                                         &dparent, parent_mode,
+                                         MDS_INODELOCK_UPDATE,
+                                         use_parent ? NULL : rec->ur_name,
+                                         rec->ur_namelen,
+                                         (rec->ur_flags & MDS_OPEN_LOCK) ?
+                                                child_lockh : NULL,
+                                         &dchild, child_mode,
+                                         MDS_INODELOCK_LOOKUP |
+                                         MDS_INODELOCK_OPEN);
+
+        if (rc) {
                 if (rc != -ENOENT) {
-                        CERROR("parent "LPU64"/%u lookup error %d\n",
+                        CERROR("parent "LPU64"/%u lookup/take lock error %d\n",
                                rec->ur_fid1->id, rec->ur_fid1->generation, rc);
                 } else {
                         /* Just cannot find parent - make it look like
@@ -974,32 +1063,13 @@ int mds_open(struct mds_update_record *rec, int offset,
 
         cleanup_phase = 1; /* parent dentry and lock */
 
-        if (rec->ur_flags & MDS_OPEN_JOIN_FILE) {
+        if (use_parent) {
                 dchild = dget(dparent);
-                cleanup_phase = 2; /* child dentry */
-                acc_mode = accmode(dchild->d_inode, rec->ur_flags);
-                GOTO(found_child, rc);
         }
 
-        /* Step 2: Lookup the child */
-      
-        if (!(lustre_msg_get_flags(req->rq_reqmsg) & MSG_REPLAY) &&
-            (rec->ur_flags & MDS_OPEN_LOCK) && (rec->ur_namelen == 1)) {
-                /* hack for nfsd with no_subtree_check, it will use anon
-                 * dentry w/o filename to open the file. the anon dentry's
-                 * parent was set to itself, so rec->ur_fid1 is the file.
-                 * And in MDC it cannot derive the dentry's parent dentry,
-                 * hence the file's name, so we hack here in MDS, 
-                 * refer to bug 13030. */
-                dchild = mds_fid2dentry(mds, rec->ur_fid1, NULL);
-        } else {
-                dchild = ll_lookup_one_len(rec->ur_name, dparent,
-                                           rec->ur_namelen - 1);
-        }
-        if (IS_ERR(dchild)) {
-                rc = PTR_ERR(dchild);
-                dchild = NULL; /* don't confuse mds_finish_transno() below */
-                GOTO(cleanup, rc);
+        if (rec->ur_flags & MDS_OPEN_JOIN_FILE) {
+                acc_mode = accmode(dchild->d_inode, rec->ur_flags);
+                GOTO(found_child, rc);
         }
 
         cleanup_phase = 2; /* child dentry */
@@ -1030,9 +1100,12 @@ int mds_open(struct mds_update_record *rec, int offset,
                         gid = current->fsgid;
 
                 /* we try to get enough quota to write here, and let ldiskfs
-                 * decide if it is out of quota or not b=14783 */
+                 * decide if it is out of quota or not b=14783
+                 * FIXME: after CMD is used, pointer to obd_trans_info* couldn't
+                 * be NULL, b=14840 */
                 lquota_chkquota(mds_quota_interface_ref, obd,
-                                current->fsuid, gid, 1, &rec_pending);
+                                current->fsuid, gid, 1, &rec_pending,
+                                NULL, NULL, 0);
 
                 intent_set_disposition(rep, DISP_OPEN_CREATE);
                 handle = fsfilt_start(obd, dparent->d_inode, FSFILT_OP_CREATE,
@@ -1046,7 +1119,9 @@ int mds_open(struct mds_update_record *rec, int offset,
                 dp.ldp_ptr = req;
                 dp.ldp_inum = ino;
 
+                LOCK_INODE_MUTEX(dparent->d_inode);
                 rc = ll_vfs_create(dparent->d_inode, dchild, rec->ur_mode,NULL);
+                UNLOCK_INODE_MUTEX(dparent->d_inode);
                 if (dchild->d_fsdata == (void *)(unsigned long)ino)
                         dchild->d_fsdata = NULL;
 
@@ -1055,15 +1130,16 @@ int mds_open(struct mds_update_record *rec, int offset,
                         GOTO(cleanup, rc);
                 }
                 inode = dchild->d_inode;
+                created = 1;
                 if (ino) {
-                        LASSERT(ino == inode->i_ino);
+                        if (ino != inode->i_ino)
+                                GOTO(cleanup, rc = -EFAULT);
                         /* Written as part of setattr */
                         inode->i_generation = rec->ur_fid2->generation;
                         CDEBUG(D_HA, "recreated ino %lu with gen %u\n",
                                inode->i_ino, inode->i_generation);
                 }
 
-                created = 1;
                 LTIME_S(iattr.ia_atime) = rec->ur_time;
                 LTIME_S(iattr.ia_ctime) = rec->ur_time;
                 LTIME_S(iattr.ia_mtime) = rec->ur_time;
@@ -1089,6 +1165,14 @@ int mds_open(struct mds_update_record *rec, int offset,
                 acc_mode = 0;           /* Don't check for permissions */
         } else {
                 acc_mode = accmode(dchild->d_inode, rec->ur_flags);
+                /* Child previously existed so the lookup and lock is already
+                 * done, so no further locking is needed. */
+                /* for nfs and join - we need two locks for same fid, but
+                 * with different mode */
+                if (need_open_lock && !use_parent)  {
+                        intent_set_disposition(rep, DISP_OPEN_LOCK);
+                        need_open_lock = 0;
+                }
         }
 
         LASSERTF(!mds_inode_is_orphan(dchild->d_inode),
@@ -1098,6 +1182,8 @@ int mds_open(struct mds_update_record *rec, int offset,
 
 found_child:
         mds_pack_inode2fid(&body->fid1, dchild->d_inode);
+        if (obd->obd_self_export->exp_bflag)
+                body->flags |= MDS_BFLAG_EXT_FLAGS;
         mds_pack_inode2body(body, dchild->d_inode);
 
         if (S_ISREG(dchild->d_inode->i_mode)) {
@@ -1152,28 +1238,18 @@ found_child:
                 GOTO(cleanup, rc = -EAGAIN);
         }
 
-        /* Obtain OPEN lock as well */
-        policy.l_inodebits.bits |= MDS_INODELOCK_OPEN;
+        if (need_open_lock) {
+                ldlm_policy_data_t policy = { .l_inodebits = { MDS_INODELOCK_LOOKUP | MDS_INODELOCK_OPEN } };
+                struct ldlm_res_id child_res_id;
 
-        /* We cannot use acc_mode here, because it is zeroed in case of
-           creating a file, so we get wrong lockmode */
-        if (rec->ur_flags & FMODE_WRITE)
-                child_mode = LCK_CW;
-        else if (rec->ur_flags & MDS_FMODE_EXEC)
-                child_mode = LCK_PR;
-        else
-                child_mode = LCK_CR;
-
-        if (!(lustre_msg_get_flags(req->rq_reqmsg) & MSG_REPLAY) && 
-             (rec->ur_flags & MDS_OPEN_LOCK)) {
                 /* In case of replay we do not get a lock assuming that the
                    caller has it already */
                 child_res_id.name[0] = dchild->d_inode->i_ino;
                 child_res_id.name[1] = dchild->d_inode->i_generation;
 
                 rc = ldlm_cli_enqueue_local(obd->obd_namespace, &child_res_id,
-                                            LDLM_IBITS, &policy, child_mode, 
-                                            &lock_flags, ldlm_blocking_ast, 
+                                            LDLM_IBITS, &policy, child_mode,
+                                            &lock_flags, ldlm_blocking_ast,
                                             ldlm_completion_ast, NULL, NULL,
                                             0, NULL, child_lockh);
                 if (rc != ELDLM_OK)
@@ -1181,7 +1257,6 @@ found_child:
 
                 /* Let mds_intent_policy know that we have a lock to return */
                 intent_set_disposition(rep, DISP_OPEN_LOCK);
-                cleanup_phase = 3;
         }
 
         if (!S_ISREG(dchild->d_inode->i_mode) &&
@@ -1204,16 +1279,15 @@ found_child:
  cleanup_no_trans:
         if (rec_pending)
                 lquota_pending_commit(mds_quota_interface_ref, obd,
-                                      current->fsuid, gid, 1);
+                                      current->fsuid, gid, rec_pending);
         switch (cleanup_phase) {
-        case 3:
-                if (rc)
-                        /* It is safe to leave IT_OPEN_LOCK set, if rc is not 0,
-                         * mds_intent_policy won't try to return any locks */
-                        ldlm_lock_decref(child_lockh, child_mode);
         case 2:
                 if (rc && created) {
-                        int err = vfs_unlink(dparent->d_inode, dchild);
+                        int err;
+                        LOCK_INODE_MUTEX(dparent->d_inode);
+                        err = ll_vfs_unlink(dparent->d_inode, dchild,
+                                            mds->mds_vfsmnt);
+                        UNLOCK_INODE_MUTEX(dparent->d_inode);
                         if (err) {
                                 CERROR("unlink(%.*s) in error path: %d\n",
                                        dchild->d_name.len, dchild->d_name.name,
@@ -1225,8 +1299,14 @@ found_child:
                         qpids[USRQUOTA] = dparent->d_inode->i_uid;
                         qpids[GRPQUOTA] = dparent->d_inode->i_gid;
                 }
-                l_dput(dchild);
         case 1:
+                if (dchild) {
+                        l_dput(dchild);
+                        /* It is safe to leave IT_OPEN_LOCK set, if rc is not 0,
+                         * mds_intent_policy won't try to return any locks */
+                        if (rc && child_lockh->cookie)
+                                ldlm_lock_decref(child_lockh, child_mode);
+                }
                 if (dparent == NULL)
                         break;
 
@@ -1282,8 +1362,7 @@ int mds_mfd_close(struct ptlrpc_request *req, int offset,
                inode->i_nlink, mds_orphan_open_count(inode));
 
         last_orphan = mds_orphan_open_dec_test(inode) &&
-                mds_inode_is_orphan(inode);
-        MDS_UP_WRITE_ORPHAN_SEM(inode);
+                      mds_inode_is_orphan(inode);
 
         /* this is half of the actual "close" */
         if (mfd->mfd_mode & FMODE_WRITE) {
@@ -1292,10 +1371,15 @@ int mds_mfd_close(struct ptlrpc_request *req, int offset,
         } else if (mfd->mfd_mode & MDS_FMODE_EXEC) {
                 mds_allow_write_access(inode);
         }
+        /* here writecount change also needs protection from orphan write sem.
+         * so drop orphan write sem after mds_put_write_access, bz 12888. */
+        MDS_UP_WRITE_ORPHAN_SEM(inode);
 
         if (last_orphan && unlink_orphan) {
                 int stripe_count = 0;
-                LASSERT(rc == 0); /* mds_put_write_access must have succeeded */
+                /* mds_put_write_access must have succeeded */
+                LASSERTF(rc == 0, "inode %lu/%u: rc %d",
+                         inode->i_ino, inode->i_generation, rc);
 
                 CDEBUG(D_INODE, "destroying orphan object %s\n", fidname);
 
@@ -1318,7 +1402,8 @@ int mds_mfd_close(struct ptlrpc_request *req, int offset,
 
                 cleanup_phase = 2; /* dput(pending_child) when finished */
                 if (S_ISDIR(pending_child->d_inode->i_mode)) {
-                        rc = vfs_rmdir(pending_dir, pending_child);
+                        rc = ll_vfs_rmdir(pending_dir, pending_child,
+                                          mds->mds_vfsmnt);
                         if (rc)
                                 CERROR("error unlinking orphan dir %s: rc %d\n",
                                        fidname,rc);
@@ -1345,52 +1430,51 @@ int mds_mfd_close(struct ptlrpc_request *req, int offset,
                 dp.ldp_inum = 0;
                 dp.ldp_ptr = req;
                 pending_child->d_fsdata = (void *) &dp;
-                rc = vfs_unlink(pending_dir, pending_child);
+                rc = ll_vfs_unlink(pending_dir, pending_child, mds->mds_vfsmnt);
                 if (rc)
                         CERROR("error unlinking orphan %s: rc %d\n",fidname,rc);
 
                 goto out; /* Don't bother updating attrs on unlinked inode */
         }
 
-#if 0
-        if (request_body != NULL && mfd->mfd_mode & FMODE_WRITE && rc == 0) {
-                /* Update the on-disk attributes if this was the last write
-                 * close, and all information was provided (i.e., rc == 0)
-                 *
-                 * XXX this should probably be abstracted with mds_reint_setattr
-                 */
+        if (request_body != NULL) {
+               if ((request_body->valid & OBD_MD_FLCTIME) &&
+                   (request_body->ctime > LTIME_S(inode->i_ctime))) {
+                       LTIME_S(iattr.ia_ctime) = request_body->ctime;
+                       iattr.ia_valid |= ATTR_CTIME;
+               }
 
-                if (request_body->valid & OBD_MD_FLMTIME &&
-                    LTIME_S(iattr.ia_mtime) > LTIME_S(inode->i_mtime)) {
-                        LTIME_S(iattr.ia_mtime) = request_body->mtime;
-                        iattr.ia_valid |= ATTR_MTIME;
-                }
-                if (request_body->valid & OBD_MD_FLCTIME &&
-                    LTIME_S(iattr.ia_ctime) > LTIME_S(inode->i_ctime)) {
-                        LTIME_S(iattr.ia_ctime) = request_body->ctime;
-                        iattr.ia_valid |= ATTR_CTIME;
-                }
+               if ((request_body->valid & OBD_MD_FLMTIME) &&
+                   (request_body->mtime > LTIME_S(inode->i_mtime)) &&
+                   (iattr.ia_valid & ATTR_CTIME)) {
+                       LTIME_S(iattr.ia_mtime) = request_body->mtime;
+                       iattr.ia_valid |= ATTR_MTIME;
+               }
 
-                /* XXX can't set block count with fsfilt_setattr (!) */
-                if (request_body->valid & OBD_MD_FLSIZE) {
-                        iattr.ia_valid |= ATTR_SIZE;
-                        iattr.ia_size = request_body->size;
-                }
-                /* iattr.ia_blocks = request_body->blocks */
-
-        }
-#endif
-        if (request_body != NULL && request_body->valid & OBD_MD_FLATIME) {
-                /* Only start a transaction to write out only the atime if
-                 * it is more out-of-date than the specified limit.  If we
-                 * are already going to write out the atime then do it anyway.
-                 * */
-                LTIME_S(iattr.ia_atime) = request_body->atime;
-                if ((LTIME_S(iattr.ia_atime) >
+               /* Only start a transaction to write out only the atime
+                * if it is more out-of-date than the specified limit.
+                * If we are already going to write out the inode then
+                * update the atime anyway.
+                */
+               if ((request_body->valid & OBD_MD_FLATIME) &&
+                   ((request_body->atime >
                      LTIME_S(inode->i_atime) + mds->mds_atime_diff) ||
                     (iattr.ia_valid != 0 &&
-                     LTIME_S(iattr.ia_atime) > LTIME_S(inode->i_atime)))
-                        iattr.ia_valid |= ATTR_ATIME;
+                     request_body->atime > LTIME_S(inode->i_atime)))) {
+                       LTIME_S(iattr.ia_atime) = request_body->atime;
+                       iattr.ia_valid |= ATTR_ATIME;
+               }
+
+               /* Store a rough estimate of the file size on the MDS for
+                * tools like e2scan and HSM that are just using this for
+                * rough decision making and will get the proper size later.
+                * This is NOT guaranteed to be correct with multiple
+                * writers, but is only needed until SOM is done. b=11063 */
+               if ((request_body->valid & OBD_MD_FLSIZE) &&
+                   (iattr.ia_valid != 0)) {
+                       iattr.ia_size = request_body->size;
+                       iattr.ia_valid |= ATTR_SIZE;
+               }
         }
 
         if (iattr.ia_valid != 0) {
