@@ -1,26 +1,43 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (C) 2002 Cluster File Systems, Inc.
- *   Author: Peter J. Braam <braam@clusterfs.com>
- *   Author: Phil Schwan <phil@clusterfs.com>
- *   Author: Robert Read <rread@clusterfs.com>
+ * GPL HEADER START
  *
- *   This file is part of Lustre, http://www.lustre.org.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   Lustre is free software; you can redistribute it and/or
- *   modify it under the terms of version 2 of the GNU General Public
- *   License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   Lustre is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   You should have received a copy of the GNU General Public License
- *   along with Lustre; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/utils/lfs.c
+ *
+ * Author: Peter J. Braam <braam@clusterfs.com>
+ * Author: Phil Schwan <phil@clusterfs.com>
+ * Author: Robert Read <rread@clusterfs.com>
  */
 
 #include <stdlib.h>
@@ -37,6 +54,9 @@
 #include <dirent.h>
 #include <time.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_QUOTA_H
+#include <sys/quota.h>
+#endif
 
 #include <lnet/api-support.h>
 #include <lnet/lnetctl.h>
@@ -59,7 +79,7 @@ static int lfs_osts(int argc, char **argv);
 static int lfs_df(int argc, char **argv);
 static int lfs_check(int argc, char **argv);
 static int lfs_catinfo(int argc, char **argv);
-#ifdef HAVE_QUOTA_SUPPORT
+#ifdef HAVE_SYS_QUOTA_H
 static int lfs_quotachown(int argc, char **argv);
 static int lfs_quotacheck(int argc, char **argv);
 static int lfs_quotaon(int argc, char **argv);
@@ -97,7 +117,9 @@ command_t cmdlist[] = {
          "usage: find <dir/file> ... \n"
          "     [[!] --atime|-A [+-]N] [[!] --mtime|-M [+-]N] [[!] --ctime|-C [+-]N]\n"
          "     [--maxdepth|-D N] [[!] --name|-n <pattern>] [--print0|-P]\n"
-         "     [--print|-p] [--obd|-O <uuid>] [[!] --type|-t <filetype>]\n"
+         "     [--print|-p] [--obd|-O <uuid[s]>] [[!] --size|-s [+-]N[bkMGTP]]\n"
+         "     [[!] --type|-t <filetype>] [[!] --gid|-g N] [[!] --group|-G <name>]\n"
+         "     [[!] --uid|-u N] [[!] --user|-U <name>]\n"
          "\t !: used before an option indicates 'NOT' the requested attribute\n"
          "\t -: used before an value indicates 'AT MOST' the requested value\n"
          "\t +: used before an option indicates 'AT LEAST' the requested value\n"},
@@ -118,7 +140,7 @@ command_t cmdlist[] = {
          "report filesystem disk space usage or inodes usage"
          "of each MDS/OSD.\n"
          "Usage: df [-i] [-h] [path]"},
-#ifdef HAVE_QUOTA_SUPPORT
+#ifdef HAVE_SYS_QUOTA_H
         {"quotachown",lfs_quotachown, 0,
          "Change files' owner or group on the specified filesystem.\n"
          "usage: quotachown [-i] <filesystem>\n"
@@ -133,7 +155,19 @@ command_t cmdlist[] = {
          "usage: quotaoff [ -ug ] <filesystem>"},
         {"setquota", lfs_setquota, 0, "Set filesystem quotas.\n"
          "usage: setquota [ -u | -g ] <name> <block-softlimit> <block-hardlimit> <inode-softlimit> <inode-hardlimit> <filesystem>\n"
-         "       setquota -t [ -u | -g ] <block-grace> <inode-grace> <filesystem>"},
+         "       setquota -t [ -u | -g ] <block-grace> <inode-grace> <filesystem>\n"
+         "       setquota [ -u | --user | -g | --group ] <name>\n"
+         "                [--block-softlimit <block-softlimit>]\n"
+         "                [--block-hardlimit <block-hardlimit>]\n"
+         "                [--inode-softlimit <inode-softlimit>]\n"
+         "                [--inode-hardlimit <inode-hardlimit>] <filesystem>\n"
+         "       setquota [-t] [ -u | --user | -g | --group ]\n"
+         "                [--block-grace <block-grace>]\n"
+         "                [--inode-grace <inode-grace>] <filesystem>\n"
+         "       -b can be used instead of --block-softlimit/--block-grace\n"
+         "       -B can be used instead of --block-hardlimit\n"
+         "       -i can be used instead of --inode-softlimit/--inode-grace\n"
+         "       -I can be used instead of --inode-hardlimit"},
         {"quota", lfs_quota, 0, "Display disk usage and limits.\n"
          "usage: quota [ -o obd_uuid ] [{-u|-g  <name>}|-t] <filesystem>"},
         {"quotainv", lfs_quotainv, 0, "Invalidate quota data.\n"
@@ -242,7 +276,7 @@ static int lfs_setstripe(int argc, char **argv)
 
         /* get the stripe size */
         if (stripe_size_arg != NULL) {
-                result = parse_size(stripe_size_arg, &st_size, &size_units);
+                result = parse_size(stripe_size_arg, &st_size, &size_units, 0);
                 if (result) {
                         fprintf(stderr,"error: bad size '%s'\n",
                                 stripe_size_arg);
@@ -301,6 +335,63 @@ static int set_time(time_t *time, time_t *set, char *str)
         return res;
 }
 
+#define USER 0
+#define GROUP 1
+
+static int name2id(unsigned int *id, char *name, int type)
+{
+        if (type == USER) {
+                struct passwd *entry;
+
+                if (!(entry = getpwnam(name))) {
+                        if (!errno)
+                                errno = ENOENT;
+                        return -1;
+                }
+
+                *id = entry->pw_uid;
+        } else {
+                struct group *entry;
+
+                if (!(entry = getgrnam(name))) {
+                        if (!errno)
+                                errno = ENOENT;
+                        return -1;
+                }
+
+                *id = entry->gr_gid;
+        }
+
+        return 0;
+}
+
+static int id2name(char **name, unsigned int id, int type)
+{
+        if (type == USER) {
+                struct passwd *entry;
+
+                if (!(entry = getpwuid(id))) {
+                        if (!errno)
+                                errno = ENOENT;
+                        return -1;
+                }
+
+                *name = entry->pw_name;
+        } else {
+                struct group *entry;
+
+                if (!(entry = getgrgid(id))) {
+                        if (!errno)
+                                errno = ENOENT;
+                        return -1;
+                }
+
+                *name = entry->gr_name;
+        }
+
+        return 0;
+}
+
 static int lfs_find(int argc, char **argv)
 {
         int new_fashion = 1;
@@ -314,6 +405,10 @@ static int lfs_find(int argc, char **argv)
                 {"ctime",     required_argument, 0, 'C'},
                 {"mtime",     required_argument, 0, 'M'},
                 {"maxdepth",  required_argument, 0, 'D'},
+                {"gid",       required_argument, 0, 'g'},
+                {"group",     required_argument, 0, 'G'},
+                {"uid",       required_argument, 0, 'u'},
+                {"user",      required_argument, 0, 'U'},
                 {"name",      required_argument, 0, 'n'},
                 /* --obd is considered as a new option. */
                 {"obd",       required_argument, 0, 'O'},
@@ -334,11 +429,12 @@ static int lfs_find(int argc, char **argv)
         time_t *xtime;
         int *xsign;
         int isoption;
+        char *endptr;
 
         time(&t);
 
         optind = 0;
-        while ((c = getopt_long_only(argc, argv, "-A:C:D:M:n:PpO:qrs:t:v",
+        while ((c = getopt_long_only(argc, argv, "-A:C:D:g:G:M:n:PpO:qrs:t:u:U:v",
                                      long_opts, NULL)) >= 0) {
                 xtime = NULL;
                 xsign = NULL;
@@ -405,6 +501,54 @@ static int lfs_find(int argc, char **argv)
                         new_fashion = 1;
                         param.maxdepth = strtol(optarg, 0, 0);
                         break;
+                case 'g':
+                        new_fashion = 1;
+                        param.gid = strtol(optarg, &endptr, 10);
+                        if (optarg == endptr) {
+                                fprintf(stderr, "Bad gid: %s\n", optarg);
+                                return CMD_HELP;
+                        }
+                        param.exclude_gid = !!neg_opt;
+                        param.check_gid = 1;
+                        break;
+                case 'G':
+                        new_fashion = 1;
+                        param.gid = strtol(optarg, &endptr, 10);
+                        if (optarg == endptr) {
+	                        ret = name2id(&param.gid, optarg, GROUP);
+        	                if (ret != 0) {
+                                        fprintf(stderr, "Group/GID: %s cannot "
+                                                "be found.\n", optarg);
+                                        return -1;
+                                }
+                        }
+                        param.exclude_gid = !!neg_opt;
+                        param.check_gid = 1;
+                        break;
+                case 'u':
+                        new_fashion = 1;
+                        param.uid = strtol(optarg, &endptr, 10);
+                        if (optarg == endptr) {
+                                fprintf(stderr, "Bad uid: %s\n", optarg);
+                                return CMD_HELP;
+                        }
+                        param.exclude_uid = !!neg_opt;
+                        param.check_uid = 1;
+                        break;
+                case 'U':
+                        new_fashion = 1;
+                        param.uid = strtol(optarg, &endptr, 10);
+                        if (optarg == endptr) {
+	                        ret = name2id(&param.uid, optarg, USER);
+        	                if (ret != 0) {
+                                        fprintf(stderr, "User/UID: %s cannot "
+                                                "be found.\n", optarg);
+                                        return -1;
+                                }
+                        }
+                        param.exclude_uid = !!neg_opt;
+                        param.check_uid = 1;
+                        break;
                 case 'n':
                         new_fashion = 1;
                         param.pattern = (char *)optarg;
@@ -496,7 +640,8 @@ static int lfs_find(int argc, char **argv)
 
                         if (param.size_sign)
                                 optarg++;
-                        ret = parse_size(optarg, &param.size,&param.size_units);
+                        ret = parse_size(optarg, &param.size,
+                                         &param.size_units, 0);
                         if (ret) {
                                 fprintf(stderr,"error: bad size '%s'\n",
                                         optarg);
@@ -622,7 +767,6 @@ static int lfs_osts(int argc, char **argv)
 {
         FILE *fp;
         struct mntent *mnt = NULL;
-        struct obd_uuid *obduuid = NULL;
         struct find_param param;
         int rc=0;
 
@@ -636,9 +780,8 @@ static int lfs_osts(int argc, char **argv)
                         strerror (errno));
         } else {
                 mnt = getmntent(fp);
-                memset(&param, 0, sizeof(param));
-                param.obduuid = obduuid;
                 while (feof(fp) == 0 && ferror(fp) ==0) {
+                        memset(&param, 0, sizeof(param));
                         if (llapi_is_lustre_mnt(mnt)) {
                                 rc = llapi_getstripe(mnt->mnt_dir, &param);
                                 if (rc)
@@ -869,7 +1012,7 @@ static int lfs_df(int argc, char **argv)
         FILE *fp;
         char *path = NULL;
         struct mntent *mnt = NULL;
-        char mntdir[PATH_MAX] = {'\0'};
+        char *mntdir = NULL;
         int ishow = 0, cooked = 0;
         int c, rc = 0;
 
@@ -896,10 +1039,19 @@ static int lfs_df(int argc, char **argv)
                         argv[0], MOUNTED, strerror(errno));
                 return rc;
         }
+
+        if ((mntdir = malloc(PATH_MAX)) == NULL) {
+                fprintf(stderr, "error: cannot allocate %d bytes\n",
+                        PATH_MAX);
+                return -ENOMEM;
+        }
+        memset(mntdir, 0, PATH_MAX);
+
         if (path) {
-                rc = path2mnt(path, fp, mntdir, sizeof(mntdir));
+                rc = path2mnt(path, fp, mntdir, PATH_MAX);
                 if (rc) {
                         endmntent(fp);
+                        free(mntdir);
                         return rc;
                 }
 
@@ -920,6 +1072,7 @@ static int lfs_df(int argc, char **argv)
                 endmntent(fp);
         }
 
+        free(mntdir);
         return rc;
 }
 
@@ -1054,7 +1207,7 @@ out:
         return rc;
 }
 
-#ifdef HAVE_QUOTA_SUPPORT
+#ifdef HAVE_SYS_QUOTA_H
 static int lfs_quotachown(int argc, char **argv)
 {
 
@@ -1081,7 +1234,6 @@ static int lfs_quotachown(int argc, char **argv)
         return rc;
 }
 
-
 static int lfs_quotacheck(int argc, char **argv)
 {
         int c, check_type = 0;
@@ -1121,8 +1273,9 @@ static int lfs_quotacheck(int argc, char **argv)
 
         memset(&qctl, 0, sizeof(qctl));
         qctl.qc_cmd = LUSTRE_Q_QUOTAOFF;
-        qctl.qc_id = QFMT_LDISKFS;
         qctl.qc_type = check_type;
+        qctl.qc_id = QFMT_LDISKFS; /* compatibility: 1.6.5 and earliers
+                                    * take this parameter into account */
         rc = llapi_quotactl(mnt, &qctl);
         if (rc) {
                 fprintf(stderr, "quota off failed: %s\n", strerror(errno));
@@ -1146,8 +1299,9 @@ static int lfs_quotacheck(int argc, char **argv)
 
         memset(&qctl, 0, sizeof(qctl));
         qctl.qc_cmd = LUSTRE_Q_QUOTAON;
-        qctl.qc_id = QFMT_LDISKFS;
         qctl.qc_type = check_type;
+        qctl.qc_id = QFMT_LDISKFS; /* compatibility: 1.6.5 and earliers
+                                    * take this parameter into account */
         rc = llapi_quotactl(mnt, &qctl);
         if (rc) {
                 if (*obd_type)
@@ -1171,7 +1325,8 @@ static int lfs_quotaon(int argc, char **argv)
 
         memset(&qctl, 0, sizeof(qctl));
         qctl.qc_cmd = LUSTRE_Q_QUOTAON;
-        qctl.qc_id = QFMT_LDISKFS;
+        qctl.qc_id = QFMT_LDISKFS; /* compatibility: 1.6.5 and earliers
+                                    * take this parameter into account */
 
         optind = 0;
         while ((c = getopt(argc, argv, "ugf")) != -1) {
@@ -1279,7 +1434,7 @@ static int lfs_quotainv(int argc, char **argv)
         qctl.qc_cmd = LUSTRE_Q_INVALIDATE;
 
         optind = 0;
-        while ((c = getopt(argc, argv, "ug")) != -1) {
+        while ((c = getopt(argc, argv, "ugf")) != -1) {
                 switch (c) {
                 case 'u':
                         qctl.qc_type |= 0x01;
@@ -1287,6 +1442,9 @@ static int lfs_quotainv(int argc, char **argv)
                 case 'g':
                         qctl.qc_type |= 0x02;
                         break;
+                case 'f':
+                        qctl.qc_cmd = LUSTRE_Q_FINVALIDATE;
+                        break;
                 default:
                         fprintf(stderr, "error: %s: option '-%c' "
                                         "unrecognized\n", argv[0], c);
@@ -1313,60 +1471,6 @@ static int lfs_quotainv(int argc, char **argv)
         return 0;
 }
 
-static int name2id(unsigned int *id, char *name, int type)
-{
-        if (type == USRQUOTA) {
-                struct passwd *entry;
-
-                if (!(entry = getpwnam(name))) {
-                        if (!errno)
-                                errno = ENOENT;
-                        return -1;
-                }
-
-                *id = entry->pw_uid;
-        } else {
-                struct group *entry;
-
-                if (!(entry = getgrnam(name))) {
-                        if (!errno)
-                                errno = ENOENT;
-                        return -1;
-                }
-
-                *id = entry->gr_gid;
-        }
-
-        return 0;
-}
-
-static int id2name(char **name, unsigned int id, int type)
-{
-        if (type == USRQUOTA) {
-                struct passwd *entry;
-
-                if (!(entry = getpwuid(id))) {
-                        if (!errno)
-                                errno = ENOENT;
-                        return -1;
-                }
-
-                *name = entry->pw_name;
-        } else {
-                struct group *entry;
-
-                if (!(entry = getgrgid(id))) {
-                        if (!errno)
-                                errno = ENOENT;
-                        return -1;
-                }
-
-                *name = entry->gr_name;
-        }
-
-        return 0;
-}
-
 #define ARG2INT(nr, str, msg)                                           \
 do {                                                                    \
         char *endp;                                                     \
@@ -1436,93 +1540,280 @@ error:
         return ULONG_MAX;
 }
 
-#define ARG2ULL(nr, str, msg)                                           \
+#define ARG2ULL(nr, str, defscale)                                      \
 do {                                                                    \
-        char *endp;                                                     \
-        nr = strtoull(str, &endp, 0);                                   \
-        if (*endp) {                                                    \
-                fprintf(stderr, "error: bad %s: %s\n", msg, str);       \
+        unsigned long long limit, units = 0;                            \
+        int rc;                                                         \
+                                                                        \
+        rc = parse_size(str, &limit, &units, 1);                        \
+        if (rc < 0) {                                                   \
+                fprintf(stderr, "error: bad limit value %s\n", str);    \
                 return CMD_HELP;                                        \
         }                                                               \
+        nr = ((units == 0) ? (defscale) : 1) * limit;                   \
 } while (0)
 
-
-int lfs_setquota(int argc, char **argv)
+static inline int has_times_option(int argc, char **argv)
 {
-        int c;
-        char *mnt;
+        int i;
+
+        for (i = 1; i < argc; i++)
+                if (!strcmp(argv[i], "-t"))
+                        return 1;
+
+        return 0;
+}
+
+int lfs_setquota_times(int argc, char **argv)
+{
+        int c, rc;
         struct if_quotactl qctl;
-        char *obd_type = (char *)qctl.obd_type;
-        int rc;
+        char *mnt, *obd_type = (char *)qctl.obd_type;
+        struct obd_dqblk *dqb = &qctl.qc_dqblk;
+        struct obd_dqinfo *dqi = &qctl.qc_dqinfo;
+        struct option long_opts[] = {
+                {"user",            no_argument,       0, 'u'},
+                {"group",           no_argument,       0, 'g'},
+                {"block-grace",     required_argument, 0, 'b'},
+                {"inode-grace",     required_argument, 0, 'i'},
+                {"times",           no_argument,       0, 't'},
+                {0, 0, 0, 0}
+        };
 
         memset(&qctl, 0, sizeof(qctl));
-        qctl.qc_cmd = LUSTRE_Q_SETQUOTA;
+        qctl.qc_cmd  = LUSTRE_Q_SETINFO;
+        qctl.qc_type = UGQUOTA;
+
+#if 1
+        /* compatibility syntax: setquota -t -[u|g] t1 t2 mnt */
+        if (argc == 6 && !strcmp(argv[1], "-t") &&
+            (!strcmp(argv[2], "-u") || !strcmp(argv[2], "-g")) &&
+            argv[3][0] != '-' && argv[4][0] != '-') {
+                fprintf(stderr, "warning: using compatibility syntax, it may not"
+                                " be available in future releases!\n");
+
+                qctl.qc_type = !strcmp(argv[2], "-u") ? USRQUOTA : GRPQUOTA;
+
+                if ((dqi->dqi_bgrace = str2sec(argv[3])) == ULONG_MAX) {
+                        fprintf(stderr, "error: bad block-grace: %s\n", argv[3]);
+                        return CMD_HELP;
+                }
+                if ((dqi->dqi_igrace = str2sec(argv[4])) == ULONG_MAX) {
+                        fprintf(stderr, "error: bad inode-grace: %s\n", argv[4]);
+                        return CMD_HELP;
+                }
+                dqb->dqb_valid = QIF_TIMES;
+                mnt = argv[argc - 1];
+                goto quotactl;
+        }
+#endif
 
         optind = 0;
-        while ((c = getopt(argc, argv, "ugt")) != -1) {
+        while ((c = getopt_long(argc, argv, "ugb:i:t", long_opts, NULL)) != -1) {
                 switch (c) {
                 case 'u':
-                        qctl.qc_type |= 0x01;
-                        break;
                 case 'g':
-                        qctl.qc_type |= 0x02;
+                        if (qctl.qc_type != UGQUOTA) {
+                                fprintf(stderr, "error: -u and -g can't be used "
+                                                "more than once\n");
+                                return CMD_HELP;
+                        }
+                        qctl.qc_type = (c == 'u') ? USRQUOTA : GRPQUOTA;
                         break;
-                case 't':
-                        qctl.qc_cmd = LUSTRE_Q_SETINFO;
+                case 'b':
+                        if ((dqi->dqi_bgrace = str2sec(optarg)) == ULONG_MAX) {
+                                fprintf(stderr, "error: bad block-grace: %s\n",
+                                        optarg);
+                                return CMD_HELP;
+                        }
+                        dqb->dqb_valid |= QIF_BTIME;
                         break;
-                default:
-                        fprintf(stderr, "error: %s: option '-%c' "
-                                        "unrecognized\n", argv[0], c);
+                case 'i':
+                        if ((dqi->dqi_igrace = str2sec(optarg)) == ULONG_MAX) {
+                                fprintf(stderr, "error: bad inode-grace: %s\n",
+                                        optarg);
+                                return CMD_HELP;
+                        }
+                        dqb->dqb_valid |= QIF_ITIME;
+                        break;
+                case 't': /* Yes, of course! */
+                        break;
+                default: /* getopt prints error message for us when opterr != 0 */
                         return CMD_HELP;
                 }
         }
 
-        if (qctl.qc_type)
-                qctl.qc_type--;
-
         if (qctl.qc_type == UGQUOTA) {
-                fprintf(stderr, "error: user and group quotas can't be set "
-                                "both\n");
+                fprintf(stderr, "error: neither -u nor -g specified\n");
                 return CMD_HELP;
         }
 
-        if (qctl.qc_cmd == LUSTRE_Q_SETQUOTA) {
-                struct obd_dqblk *dqb = &qctl.qc_dqblk;
-
-                if (optind + 6 != argc)
-                        return CMD_HELP;
-
-                rc = name2id(&qctl.qc_id, argv[optind++], qctl.qc_type);
-                if (rc) {
-                        fprintf(stderr, "error: find id for name %s failed: %s\n",
-                                argv[optind - 1], strerror(errno));
-                        return CMD_HELP;
-                }
-
-                ARG2ULL(dqb->dqb_bsoftlimit, argv[optind++], "block-softlimit");
-                ARG2ULL(dqb->dqb_bhardlimit, argv[optind++], "block-hardlimit");
-                ARG2ULL(dqb->dqb_isoftlimit, argv[optind++], "inode-softlimit");
-                ARG2ULL(dqb->dqb_ihardlimit, argv[optind++], "inode-hardlimit");
-
-                dqb->dqb_valid = QIF_LIMITS;
-        } else {
-                struct obd_dqinfo *dqi = &qctl.qc_dqinfo;
-
-                if (optind + 3 != argc)
-                        return CMD_HELP;
-
-                if ((dqi->dqi_bgrace = str2sec(argv[optind++])) == ULONG_MAX) {
-                        fprintf(stderr, "error: bad %s: %s\n", "block-grace", argv[optind - 1]);
-                        return CMD_HELP;
-                }
-                if ((dqi->dqi_igrace = str2sec(argv[optind++])) == ULONG_MAX) {
-                        fprintf(stderr, "error: bad %s: %s\n", "inode-grace", argv[optind - 1]);
-                        return CMD_HELP;
-		}
+        if (optind != argc - 1) {
+                fprintf(stderr, "error: unexpected parameters encountered\n");
+                return CMD_HELP;
         }
 
         mnt = argv[optind];
 
+#if 1
+quotactl:
+#endif
+        rc = llapi_quotactl(mnt, &qctl);
+        if (rc) {
+                if (*obd_type)
+                        fprintf(stderr, "%s %s ", obd_type,
+                                obd_uuid2str(&qctl.obd_uuid));
+                fprintf(stderr, "setquota failed: %s\n", strerror(errno));
+                return rc;
+        }
+
+        return 0;
+}
+
+#define BSLIMIT (1 << 0)
+#define BHLIMIT (1 << 1)
+#define ISLIMIT (1 << 2)
+#define IHLIMIT (1 << 3)
+
+int lfs_setquota(int argc, char **argv)
+{
+        int c, rc;
+        struct if_quotactl qctl;
+        char *mnt, *obd_type = (char *)qctl.obd_type;
+        struct obd_dqblk *dqb = &qctl.qc_dqblk;
+        struct option long_opts[] = {
+                {"user",            required_argument, 0, 'u'},
+                {"group",           required_argument, 0, 'g'},
+                {"block-softlimit", required_argument, 0, 'b'},
+                {"block-hardlimit", required_argument, 0, 'B'},
+                {"inode-softlimit", required_argument, 0, 'i'},
+                {"inode-hardlimit", required_argument, 0, 'I'},
+                {0, 0, 0, 0}
+        };
+        unsigned limit_mask = 0;
+
+        if (has_times_option(argc, argv))
+                return lfs_setquota_times(argc, argv);
+
+        memset(&qctl, 0, sizeof(qctl));
+        qctl.qc_cmd  = LUSTRE_Q_SETQUOTA;
+        qctl.qc_type = UGQUOTA; /* UGQUOTA makes no sense for setquota,
+                                 * so it can be used as a marker that qc_type
+                                 * isn't reinitialized from command line */
+
+#if 1
+        /* compatibility syntax: [-u|-g] <user|group> b B i I mount
+         * will be removed in the future */
+        if (argc == 8 && (!strcmp(argv[1], "-u") || !strcmp(argv[1], "-g")) &&
+            argv[3][0] != '-' && argv[4][0] != '-' && argv[5][0] != '-' &&
+            argv[6][0] != '-') {
+                fprintf(stderr, "warning: using compatibility syntax, it may not"
+                                " be available in future releases!\n");
+
+                qctl.qc_type = !strcmp(argv[1], "-u") ? USRQUOTA : GRPQUOTA;
+                rc = name2id(&qctl.qc_id, argv[2],
+                             (qctl.qc_type == USRQUOTA) ? USER : GROUP);
+                if (rc) {
+                        fprintf(stderr, "error: unknown id %s\n", argv[2]);
+                        return CMD_HELP;
+                }
+
+                ARG2ULL(dqb->dqb_bsoftlimit, argv[3], 1024);
+                dqb->dqb_bsoftlimit >>= 10;
+                ARG2ULL(dqb->dqb_bhardlimit, argv[4], 1024);
+                dqb->dqb_bhardlimit >>= 10;
+                ARG2ULL(dqb->dqb_isoftlimit, argv[5], 1);
+                ARG2ULL(dqb->dqb_ihardlimit, argv[6], 1);
+
+                dqb->dqb_valid = QIF_LIMITS;
+                mnt = argv[argc - 1];
+                goto quotactl;
+        }
+#endif
+
+        optind = 0;
+        while ((c = getopt_long(argc, argv, "u:g:b:B:i:I:", long_opts, NULL)) != -1) {
+                switch (c) {
+                case 'u':
+                case 'g':
+                        if (qctl.qc_type != UGQUOTA) {
+                                fprintf(stderr, "error: -u and -g can't be used"
+                                                " more than once\n");
+                                return CMD_HELP;
+                        }
+                        qctl.qc_type = (c == 'u') ? USRQUOTA : GRPQUOTA;
+                        rc = name2id(&qctl.qc_id, optarg,
+                                     (qctl.qc_type == USRQUOTA) ? USER : GROUP);
+                        if (rc) {
+                                fprintf(stderr, "error: unknown id %s\n",
+                                        optarg);
+                                return CMD_HELP;
+                        }
+                        break;
+                case 'b':
+                        ARG2ULL(dqb->dqb_bsoftlimit, optarg, 1024);
+                        dqb->dqb_bsoftlimit >>= 10;
+                        limit_mask |= BSLIMIT;
+                        break;
+                case 'B':
+                        ARG2ULL(dqb->dqb_bhardlimit, optarg, 1024);
+                        dqb->dqb_bhardlimit >>= 10;
+                        limit_mask |= BHLIMIT;
+                        break;
+                case 'i':
+                        ARG2ULL(dqb->dqb_isoftlimit, optarg, 1);
+                        limit_mask |= ISLIMIT;
+                        break;
+                case 'I':
+                        ARG2ULL(dqb->dqb_ihardlimit, optarg, 1);
+                        limit_mask |= IHLIMIT;
+                        break;
+                default: /* getopt prints error message for us when opterr != 0 */
+                        return CMD_HELP;
+                }
+        }
+
+        if (qctl.qc_type == UGQUOTA) {
+                fprintf(stderr, "error: neither -u nor -g are specified\n");
+                return CMD_HELP;
+        }
+
+        if (optind != argc - 1) {
+                fprintf(stderr, "error: unexpected parameters encountered\n");
+                return CMD_HELP;
+        }
+
+        mnt = argv[optind];
+
+        if ((!(limit_mask & BHLIMIT) ^ !(limit_mask & BSLIMIT)) ||
+            (!(limit_mask & IHLIMIT) ^ !(limit_mask & ISLIMIT))) {
+                /* sigh, we can't just set blimits/ilimits */
+                struct if_quotactl tmp_qctl = {.qc_cmd  = LUSTRE_Q_GETQUOTA,
+                                               .qc_type = qctl.qc_type,
+                                               .qc_id   = qctl.qc_id};
+
+                rc = llapi_quotactl(mnt, &tmp_qctl);
+                if (rc < 0) {
+                        fprintf(stderr, "error: getquota failed\n");
+                        return CMD_HELP;
+                }
+
+                if (!(limit_mask & BHLIMIT))
+                        dqb->dqb_bhardlimit = tmp_qctl.qc_dqblk.dqb_bhardlimit;
+                if (!(limit_mask & BSLIMIT))
+                        dqb->dqb_bsoftlimit = tmp_qctl.qc_dqblk.dqb_bsoftlimit;
+                if (!(limit_mask & IHLIMIT))
+                        dqb->dqb_ihardlimit = tmp_qctl.qc_dqblk.dqb_ihardlimit;
+                if (!(limit_mask & ISLIMIT))
+                        dqb->dqb_isoftlimit = tmp_qctl.qc_dqblk.dqb_isoftlimit;
+        }
+
+        dqb->dqb_valid |= (limit_mask & (BHLIMIT | BSLIMIT)) ? QIF_BLIMITS : 0;
+        dqb->dqb_valid |= (limit_mask & (IHLIMIT | ISLIMIT)) ? QIF_ILIMITS : 0;
+
+#if 1
+quotactl:
+#endif
         rc = llapi_quotactl(mnt, &qctl);
         if (rc) {
                 if (*obd_type)
@@ -1611,10 +1902,10 @@ static void print_quota(char *mnt, struct if_quotactl *qctl, int type)
                 struct obd_dqblk *dqb = &qctl->qc_dqblk;
 
                 if (dqb->dqb_bhardlimit &&
-                    toqb(dqb->dqb_curspace) > dqb->dqb_bhardlimit) {
+                    toqb(dqb->dqb_curspace) >= dqb->dqb_bhardlimit) {
                         bover = 1;
                 } else if (dqb->dqb_bsoftlimit &&
-                           toqb(dqb->dqb_curspace) > dqb->dqb_bsoftlimit) {
+                           toqb(dqb->dqb_curspace) >= dqb->dqb_bsoftlimit) {
                         if (dqb->dqb_btime > now) {
                                 bover = 2;
                         } else {
@@ -1623,10 +1914,10 @@ static void print_quota(char *mnt, struct if_quotactl *qctl, int type)
                 }
 
                 if (dqb->dqb_ihardlimit &&
-                    dqb->dqb_curinodes > dqb->dqb_ihardlimit) {
+                    dqb->dqb_curinodes >= dqb->dqb_ihardlimit) {
                         iover = 1;
                 } else if (dqb->dqb_isoftlimit &&
-                           dqb->dqb_curinodes > dqb->dqb_isoftlimit) {
+                           dqb->dqb_curinodes >= dqb->dqb_isoftlimit) {
                         if (dqb->dqb_btime > now) {
                                 iover = 2;
                         } else {
@@ -1772,21 +2063,30 @@ out:
 static int lfs_quota(int argc, char **argv)
 {
         int c;
-        char *name = NULL, *mnt;
+        char *mnt, *name = NULL;
         struct if_quotactl qctl = { .qc_cmd = LUSTRE_Q_GETQUOTA,
-                                    .qc_type = 0x01 };
+                                    .qc_type = UGQUOTA };
         char *obd_type = (char *)qctl.obd_type;
         char *obd_uuid = (char *)qctl.obd_uuid.uuid;
-        int rc, rc1 = 0, rc2 = 0, rc3 = 0;
+        int rc, rc1 = 0, rc2 = 0, rc3 = 0, verbose = 0;
+        int pass = 0;
 
         optind = 0;
-        while ((c = getopt(argc, argv, "ugto:")) != -1) {
+        while ((c = getopt(argc, argv, "ugto:v")) != -1) {
                 switch (c) {
                 case 'u':
-                        qctl.qc_type = 0x01;
+                        if (qctl.qc_type != UGQUOTA) {
+                                fprintf(stderr, "error: use either -u or -g\n");
+                                return CMD_HELP;
+                        }
+                        qctl.qc_type = USRQUOTA;
                         break;
                 case 'g':
-                        qctl.qc_type = 0x02;
+                        if (qctl.qc_type != UGQUOTA) {
+                                fprintf(stderr, "error: use either -u or -g\n");
+                                return CMD_HELP;
+                        }
+                        qctl.qc_type = GRPQUOTA;
                         break;
                 case 't':
                         qctl.qc_cmd = LUSTRE_Q_GETINFO;
@@ -1794,6 +2094,9 @@ static int lfs_quota(int argc, char **argv)
                 case 'o':
                         strncpy(obd_uuid, optarg, sizeof(qctl.obd_uuid));
                         break;
+                case 'v':
+                        verbose = 1;
+                        break;
                 default:
                         fprintf(stderr, "error: %s: option '-%c' "
                                         "unrecognized\n", argv[0], c);
@@ -1801,51 +2104,62 @@ static int lfs_quota(int argc, char **argv)
                 }
         }
 
-        if (qctl.qc_type)
-                qctl.qc_type--;
-
-
-        if (qctl.qc_cmd == LUSTRE_Q_GETQUOTA) {
+        /* current uid/gid info for "lfs quota /path/to/lustre/mount" */
+        if (qctl.qc_cmd == LUSTRE_Q_GETQUOTA && qctl.qc_type == UGQUOTA &&
+            optind == argc - 1) {
+ug_output:
+                memset(&qctl, 0, sizeof(qctl)); /* spoiled by print_*_quota */
+                qctl.qc_cmd = LUSTRE_Q_GETQUOTA;
+                if (pass++ == 0) {
+                        qctl.qc_type = USRQUOTA;
+                        qctl.qc_id = geteuid();
+                } else {
+                        qctl.qc_type = GRPQUOTA;
+                        qctl.qc_id = getegid();
+                }
+                rc = id2name(&name, qctl.qc_id,
+                             (qctl.qc_type == USRQUOTA) ? USER : GROUP);
+                if (rc)
+                        name = "<unknown>";
+        } else if (qctl.qc_cmd == LUSTRE_Q_GETQUOTA) {
                 if (optind + 2 != argc) {
                         fprintf(stderr, "error: missing quota argument(s)\n");
                         return CMD_HELP;
                 }
 
                 name = argv[optind++];
-                rc = name2id(&qctl.qc_id, name, qctl.qc_type);
+                rc = name2id(&qctl.qc_id, name,
+                             (qctl.qc_type == USRQUOTA) ? USER : GROUP);
                 if (rc) {
                         fprintf(stderr,"error: can't find id for name %s: %s\n",
                                 name, strerror(errno));
                         return CMD_HELP;
                 }
-                print_quota_title(name, &qctl);
         } else if (optind + 1 != argc) {
                 fprintf(stderr, "error: missing quota info argument(s)\n");
                 return CMD_HELP;
         }
 
+        if (qctl.qc_cmd == LUSTRE_Q_GETQUOTA)
+                print_quota_title(name, &qctl);
+
         mnt = argv[optind];
 
         rc1 = llapi_quotactl(mnt, &qctl);
         if (rc1 == -1 && errno == ESRCH) {
                 fprintf(stderr, "\n%s quotas are not enabled.\n", 
-                        qctl.qc_type == 0x00 ? "user" : "group");
-                return 0;
+                        qctl.qc_type == USRQUOTA ? "user" : "group");
+                goto out;
         }
         if (rc1 && *obd_type)
                 fprintf(stderr, "%s %s ", obd_type, obd_uuid);
 
-        if (!name)
-                rc = id2name(&name, getuid(), qctl.qc_type);
-
-        if (*obd_uuid) {
+        if (*obd_uuid)
                 mnt = "";
-                name = obd_uuid;
-        }
 
         print_quota(mnt, &qctl, GENERAL_QUOTA_INFO);
 
-        if (!*obd_uuid && qctl.qc_cmd != LUSTRE_Q_GETINFO) {
+        if (!*obd_uuid && qctl.qc_cmd != LUSTRE_Q_GETINFO && verbose) {
                 rc2 = print_mds_quota(mnt, &qctl);
                 rc3 = print_lov_quota(mnt, &qctl);
         }
@@ -1854,9 +2168,14 @@ static int lfs_quota(int argc, char **argv)
                 printf("Some errors happened when getting quota info. "
                        "Some devices may be not working or deactivated. "
                        "The data in \"[]\" is inaccurate.\n");
+
+out:
+        if (pass == 1)
+                goto ug_output;
+
         return 0;
 }
-#endif /* HAVE_QUOTA_SUPPORT */
+#endif /* HAVE_SYS_QUOTA_H! */
 
 int main(int argc, char **argv)
 {
